/*
 * title:       Protocol Support Routines
 *
 * author:      Craig E. Ward
 *
 * envionment:  GCC 3.1
 */
#include <stdlib.h>
#include <string.h>

#include "mytypes.h"
#include "stocks.h"
#include "protocol.h"

/* static functions */

/* public functions */

/*
 * Build a trader request message
 */
char *build_request(char *msgbuf,
                    size_t bufsize,
                    int symbol_code,
                    int request_code)
{
  log(__FILE__".build_request: bufsize=%d, symbol_code=%d, request_code=%d",
      bufsize, symbol_code, request_code);
  memset(msgbuf, '\0', bufsize);
  strcpy(msgbuf, requests[request_code]);
  log(__FILE__".build_request: added request \"%s\"", msgbuf);
  strcat(msgbuf, MSG_DELIMITER);
  log(__FILE__".build_request: added delimiter \"%s\"", msgbuf);
  strcat(msgbuf, symbols[symbol_code]);
  log(__FILE__".build_request: built \"%s\"", msgbuf);
  return msgbuf;
}

/*
 * Return the request part of a message
 */
char *get_request_part(const char *msg)
{
  static char request_part[MAX_REQUEST_LENGTH + 1];

  memset(request_part, '\0', sizeof request_part);

  int i = 0;
  while (msg[i] != MSG_DELIMITER[0]) {
    request_part[i] = msg[i];
    i++;
  }
  return request_part;
}

/*
 * Return the result part of a message
 */
char *get_result_part(const char *msg)
{
  log(__FILE__ ".get_result_part: \"%s\"", msg);
  static char result_part[MAX_RESULT_LENGTH + 1];
  char *end_request;

  memset(result_part, '\0', sizeof result_part);
  
  int i = 0;
  end_request = strstr(msg, MSG_DELIMITER_STRING);
  end_request++;                /* skip deliminter */
  while (*end_request != MSG_DELIMITER_CHAR && *end_request != NULL) {
    result_part[i++] = *end_request++;
  }
  log(__FILE__ ".get_result_part: found \"%s\"", result_part);
  return result_part;
}
/*
 * Return the Symbol part of a BUY/SELL message
 */
char *get_request_symbol_part(const char *msg)
{
  return get_result_part(msg);
}

/*
 * Return the price part of a CHECK message
 */
char *get_check_price_part(const char *msg)
{
  return get_buy_price_part(msg);
}

/*
 * Return the price part of a BUY request
 */
char *get_buy_price_part(const char *msg)
{
  static char price_part[MAX_PRICE_STRING + 1];
  char *price_begin;

  memset(price_part, '\0', sizeof price_part);
  price_begin = strrchr(msg, MSG_DELIMITER_CHAR);
  if (price_begin != NULL) {
    price_begin++;
    strncpy(price_part, price_begin, MAX_PRICE_STRING);
  } else {
    log(__FILE__ 
        ".get_buy_price_part: can't find message delimiter in message \"%s\"",
        msg);
  }
  log(__FILE__ ".get_buy_price_part: found \"%s\"", price_part);
  return price_part;
}

/*
 * Return code for a request, i.e. index into array
 */
int find_request_by_name(const char *request)
{
  log(__FILE__ ".find_request_by_name: %s", request);
  int i = 0;
  for (i = 0; i < (sizeof requests / sizeof(char *)); i++) {
    if (strcmp(request, requests[i]) == 0) {
      break;
    }
  }
  log(__FILE__ ".find_request_by_name: return %d (%s)", i, requests[i]);
  return i;
}

/*
 * Return code for a result, i.e. index into array
 */
int find_result_by_name(const char *result)
{
  log(__FILE__ ".find_result_by_name: %s", result);
  int i = 0;
  for (i = 0; i < (sizeof results / sizeof(char *)); i++) {
    if (strcmp(result, results[i]) == 0) {
      break;
    }
  }
  log(__FILE__ ".find_result_by_name: return %d (%s)", i, results[i]);
  return i;
}

/* */
