/*
 * title:       Message exchange protocol for stock market simulation
 *
 * author:      Craig E. Ward
 *
 * environment: UNIX/ANSI C (Darwin/GCC 3.1)
 *
 * description:
 *              CMSI 587 Operating Systems, Spring 2003
 *              Assignment 4 -- Interprocess Communication &
 *              Synchronication-Coordination
 *              Instructor: Tri Nguyen
 *              Due: March 19
 *
 * The format of messages is as delimited text strings.
 *
 * Trader-to-market messages:
 *      BUY:{SYMBOL}            Buy 100 shares of stock with SYMBOL
 *      SELL:{SYMBOL}           Sell 100 shares of stock with SYMBOL
 *      CHECK:{SYMBOL}          Request current unit value of SYMBOL
 * Market-to-trader messages:
 *      {REQUEST}:{STATUS}(:{unit_value}) Status of last request (BUY and CHECK
 *                                        includes the unit price of the
 *                                        stock.)
 *      DELIST:{SYMBOL}         Traders must be prepared to process this as a
 *                              reply. 
 * The STATUS codes are:
 *      YES     Trader may count transaction in protofolio
 *      NO      Trader may NOT count transaction in portfolio
 *      DELIST  Trader must remove stock from portfolio
 *      String representing current value of stock x 100
 *
 * If a trader received the DELIST message, it must remove that stock from its
 * portfolio.
 *
 * Rules of Trade:
 * (1) All trades are in lots of 100. Initial count available shares of any
 *     stock must be a multiple of 100.
 * (2) A trader may buy any stock that current has available shares in the
 *     market.
 * (3) A trader may not sell a stock not in the trader's protfolio. (The trader
 *     program must keep track of this for the simulation.)
 */
#ifndef PROTOCOL_H
#define PROTOCOL_H  1

#define MAX_REQUEST_LENGTH      8
#define MAX_RESULT_LENGTH       16
#define MAX_PRICE_STRING        MAX_RESULT_LENGTH

#define MSG_DELIMITER ":"
#define MSG_DELIMITER_STRING    MSG_DELIMITER
#define MSG_DELIMITER_CHAR      MSG_DELIMITER[0]

/* Constants for choices of options for traders */
#define BUY     0
#define SELL    1
#define CHECK   2
#define DISPLAY 3
#define SLEEP   4

/* Things to ask of the Market */
static const char *requests[] = {
  "BUY",                        /* Buy a stock */
  "SELL",                       /* Sell a stock */
  "CHECK"                       /* Price check a stock */
};

/* What the Market may tell Traders */
static const char *results[] = {
  "NO",                         /* Buy or Sell failed */
  "YES",                        /* Buy or Sell succeeded */
  "DELIST"                      /* That stock delisited */
};

char *build_request(char *,     /* buffer to write to */
                    size_t,     /* size of that buffer */
                    int,        /* code for symbol */
                    int);       /* code for request */

char *get_request_part(const char *);
char *get_result_part(const char *);
char *get_request_symbol_part(const char *);
char *get_check_price_part(const char *);
char *get_buy_price_part(const char *);
int find_request_by_name(const char *);
int find_result_by_name(const char *);

#endif /* PROTOCOL_H */
