/* 
 * title:       Wrappers for semaphore system calls
 *
 * author:      Craig E. Ward
 *
 * environment: Mac OS X (Darwin/GCC 3.1)
 *
 * description:
 *              CMSI 587 Operating Systems, Spring 2003
 *              Assignment 4 -- Interprocess Communication &
 *              Synchronication-Coordination
 *              Instructor: Tri Nguyen
 *              Due: March 19
 *
 * The Darwin implementation of semaphores is slightly different from other
 * UNIX-class operating systems, such as Linux, Solaris, etc. While these
 * functions can be used on other systems, this supporting code can not.
 *
 * The main difference seems to be in naming. The sem_open() call here is
 * called sem_init() on some other systems.
 */
#include <errno.h>
#include <string.h>

#include "util.h"
#include "semaphores.h"

/*
 * Create a semaphore
 */
sem_t *semaphore_util_create(const char *name)
{
  mode_t mode = 0644;
  sem_t *semaphore;
  log(__FILE__ ".semaphore_create: \"%s\"", name);
  semaphore = sem_open(name, O_CREAT, mode, 1);
  log(__FILE__ ".semaphore_create: after call to sem_open");
  if (semaphore == (sem_t *)SEM_FAILED) {
    log(__FILE__ ".semaphore_create: create failed: %s", strerror(errno));
  }
  log(__FILE__ ".semaphore_create: return");
  return semaphore;
}

/*
 * Mark the semaphore as available
 */
int semaphore_release(sem_t *semaphore)
{
  int status = sem_post(semaphore);
  if (status != 0) {
    log(__FILE__ ".semaphore_acquire: acquire failed: %s", strerror(errno));
  }
  return status;
}

/*
 * Acquire the semaphone, blocking if necessary
 */
int semaphore_acquire(sem_t *semaphore)
{
  int status = sem_wait(semaphore);
  if (status != 0) {
    log(__FILE__ ".semaphore_acquire: acquire failed: %s", strerror(errno));
  }
  return status;
}

/*
 * Remove the semaphore from the system
 */
int semaphore_close(sem_t *semaphore)
{
  int status = sem_close(semaphore);
  if (status != 0) {
    log(__FILE__ ".semaphore_close: acquire failed: %s", strerror(errno));
  }
  return status;
}
