/*
 * title:       Data structures supporting a stock market simulation
 *
 * author:      Craig E. Ward
 *
 * environment: GCC 3.1
 */
#ifndef STOCKS_H                /* idempotence */
#define STOCKS_H                /* any value (no value) will do */

/*
 * basic data structures
 *
 * Both traders and the market program will use linked-lists of these
 * structures to manipulate stocks from their own particular points-of-view.
 */
#define SYMBOL_LENGTH 5
struct stock {
  char symbol[SYMBOL_LENGTH];
  int code;                     /* to make comparisons easier */
  double unitvalue;             /* in cents! */
};

/* market view */
#define DELISTED_FLAG   -1
#define MIN_UNIT_VALUE  100 /* prices are in cents! */
struct listing {
  struct stock * astock;
  int available_shares;
  struct listing * next;
};

/* trader view */
struct portfolio_entry {
  struct stock * astock;
  int shares;
  struct portfolio_entry * next;
};

/* Available stocks by symbol */
/*static const char *symbols[] = {
  "ACME",
  "AJAX",
  "BSDI",
  "IAIX",
  "IBMX",
  "LINX",
  "LINX",
  "LMUU",
  "RTIS",
  "SUNM",
  "SVR4",
  "UNIX",
  "UNIX",
  "VMSX",
  "XTRW"
  }; */
static const char *symbols[] = {
  "BSDI",
  "LINX",
  "SVR4",
  "UNIX"
};
#define SYMBOL_COUNT    (sizeof symbols / (sizeof(char *)))
static const int NUMBER_SYMBOLS = SYMBOL_COUNT;

/*
 * Trader support functions
 */
struct portfolio_entry *add_stock(struct portfolio_entry *, /* trader portfolio */
                                  int,          /* symbol code */
                                  int,          /* number of shares */
                                  double);              /* unit price */

struct portfolio_entry *remove_stock(struct portfolio_entry *, /* trader portfolio */
                  int);         /* symbol code */

int update_stock_count(struct portfolio_entry *, /* trader portfolio */
                       int,     /* symbol code */
                       int);    /* share change */

void update_stock_price(struct portfolio_entry *, /* trader portfolio */
                        int,    /* symbol code */
                        double); /* unit price */

bool_t is_owned(struct portfolio_entry *, /* trader portfolio */
                int);           /* symbol code */

/*
 * Market support functions
 */
struct listing *get_stock_listing(struct listing *, /* pointer to listings */
                                  int); /* symbol_code */

/*
 * Utilities
 */

int find_stock_by_name(const char *);
#endif /* STOCKS_H */
