/*
 * title:       Profiles of Utility Routines
 *
 * author:      Craig E. Ward
 *
 * environment: UNIX, ANSI C
 */
#ifndef UTIL_H
#define UTIL_H

/*---------------------------------------------------------------------------
 * Name:        log
 *
 * Purpose:     This is a simple function that accepts a variable number of
 *              arguments and writes them to stdout. Redirect to a file to get
 *              output to a file.
 *
 * Arguments:
 *      format  : String describing format of arguments that follow.
 *
 * Return:      n/a
 *
 * Notes:       Requires stdarg.h for the variable argument list processing.
 *---------------------------------------------------------------------------
 */

void log(char *, ...);

/*---------------------------------------------------------------------------
 * Name:        random_choice
 *
 * Purpose:     Return the value generatated by random(3) modulo the argument
 *              passed to the function. The range of value returned will be
 *              between 0 and range-1 inclusive.
 *
 * Arguments:
 *      range   : Value to use in the modulo calculation
 *
 * Return:      Random integer
 *---------------------------------------------------------------------------
 */

int random_choice(int range);

/*---------------------------------------------------------------------------
 * Name:        seed_random_choice
 *
 * Purpose:     Initialize the random number generator
 *
 * Arguments:
 *      seed    : Value to use in seed initialization
 *
 * Return:      n/a
 *---------------------------------------------------------------------------
 */
void seed_random_choice(unsigned long seed);

#endif                                  /* UTIL_H */
